@@ -451,7 +451,7 @@ static int verify_nodename(struct objdb_iface_ver0 *objdb, char *node)
 	struct sockaddr *sa;
 	hdb_handle_t nodes_handle;
 	hdb_handle_t find_handle = 0;
-	int error;
+	int found = 0;
 
 	/* nodename is either from commandline or from uname */
 	if (nodelist_byname(objdb, cluster_parent_handle, node))
@@ -497,12 +497,11 @@ static int verify_nodename(struct objdb_iface_ver0 *objdb, char *node)
 	}
 	objdb->object_find_destroy(find_handle);
 
-
-	/* The cluster.conf names may not be related to uname at all,
-	   they may match a hostname on some network interface.
-	   NOTE: This is IPv4 only */
-	error = getifaddrs(&ifa_list);
-	if (error)
+	/*
+	 * The cluster.conf names may not be related to uname at all,
+	 * they may match a hostname on some network interface.
+	 */
+	if (getifaddrs(&ifa_list))
 		return -1;
 
 	for (ifa = ifa_list; ifa; ifa = ifa->ifa_next) {
@@ -521,12 +520,13 @@ static int verify_nodename(struct objdb_iface_ver0 *objdb, char *node)
 		if (sa->sa_family == AF_INET6)
 			salen = sizeof(struct sockaddr_in6);
 
-		error = getnameinfo(sa, salen, nodename2,
-				    sizeof(nodename2), NULL, 0, 0);
-		if (!error) {
+		if (getnameinfo(sa, salen,
+				nodename2, sizeof(nodename2), 
+				NULL, 0, 0) == 0) {
 
 			if (nodelist_byname(objdb, cluster_parent_handle, nodename2)) {
 				strncpy(node, nodename2, sizeof(nodename) - 1);
+				found = 1;
 				goto out;
 			}
 
@@ -537,27 +537,88 @@ static int verify_nodename(struct objdb_iface_ver0 *objdb, char *node)
 
 				if (nodelist_byname(objdb, cluster_parent_handle, nodename2)) {
 					strncpy(node, nodename2, sizeof(nodename) - 1);
+					found = 1;
 					goto out;
 				}
 			}
 		}
 
 		/* See if it's the IP address that's in cluster.conf */
-		error = getnameinfo(sa, sizeof(*sa), nodename2,
-				    sizeof(nodename2), NULL, 0, NI_NUMERICHOST);
-		if (error)
+		if (getnameinfo(sa, sizeof(*sa),
+				nodename2, sizeof(nodename2), 
+				NULL, 0, NI_NUMERICHOST))
 			continue;
 
 		if (nodelist_byname(objdb, cluster_parent_handle, nodename2)) {
 			strncpy(node, nodename2, sizeof(nodename) - 1);
+			found = 1;
 			goto out;
 		}
 	}
 
-	error = -1;
  out:
+	if (found) {
+		freeifaddrs(ifa_list);
+		return 0;
+	}
+
+	/*
+	 * This section covers the usecase where the nodename specified in cluster.conf
+	 * is an alias specified in /etc/hosts. For example:
+	 * <ipaddr> hostname alias1 alias2
+	 * and <clusternode name="alias2">
+	 * the above calls use uname and getnameinfo does not return aliases.
+	 * here we take the name specified in cluster.conf, resolve it to an address
+	 * and then compare against all known local ip addresses.
+	 * if we have a match, we found our nodename. In theory this chunk of code
+	 * could replace all the checks above, but let's avoid any possible regressions
+	 * and use it as last.
+	 */
+
+	nodes_handle = nodeslist_init(objdb, cluster_parent_handle, &find_handle);
+	while (nodes_handle) {
+		char *dbnodename = NULL;
+		struct addrinfo hints;
+		struct addrinfo *result = NULL, *rp = NULL;
+
+		if (objdb_get_string(objdb, nodes_handle, "name", &dbnodename)) {
+			goto next;
+		}
+
+		memset(&hints, 0, sizeof(struct addrinfo));
+		hints.ai_family = AF_UNSPEC;
+		hints.ai_socktype = SOCK_DGRAM;
+		hints.ai_flags = 0;
+		hints.ai_protocol = IPPROTO_UDP;
+
+		if (getaddrinfo(dbnodename, NULL, &hints, &result))
+			goto next;
+
+		for (rp = result; rp != NULL; rp = rp->ai_next) {
+			for (ifa = ifa_list; ifa; ifa = ifa->ifa_next) {
+				if (ipaddr_equal((struct sockaddr_storage *)rp->ai_addr,
+						 (struct sockaddr_storage *)ifa->ifa_addr)) {
+					freeaddrinfo(result);
+					strncpy(node, dbnodename, sizeof(nodename) - 1);
+					found = 1;
+					goto out2;
+				}
+			}
+		}
+
+		freeaddrinfo(result);
+ next:
+		nodes_handle = nodeslist_next(objdb, find_handle);
+	}
+ out2:
+	objdb->object_find_destroy(find_handle);
 	freeifaddrs(ifa_list);
-	return error;
+
+	if (found) {
+		return 0;
+	}
+
+	return -1;
 }
 
 /* Get any environment variable overrides */
@@ -806,47 +867,6 @@ static int get_nodename(struct objdb_iface_ver0 *objdb)
 		objdb_get_int(objdb, object_handle, "port", &portnum, mcast_portnum);
 	}
 
-	/* Check for broadcast */
-	if (!objdb_get_string(objdb, object_handle, "broadcast", &str)) {
-		if (strcmp(str, "yes") == 0) {
-			mcast_name = strdup("255.255.255.255");
-			if (!mcast_name)
-				return -1;
-			transport = TX_MECH_UDPB;
-		}
-	}
-
-	/* Check for transport */
-	if (!objdb_get_string(objdb, object_handle, "transport", &str)) {
-		if (strcmp(str, "udp") == 0) {
-			if (transport != TX_MECH_UDPB) {
-				transport = TX_MECH_UDP;
-			}
-		} else if (strcmp(str, "udpb") == 0) {
-			transport = TX_MECH_UDPB;
-		} else if (strcmp(str, "udpu") == 0) {
-			if (transport != TX_MECH_UDPB) {
-				transport = TX_MECH_UDPU;
-			} else {
-				snprintf(error_reason, sizeof(error_reason) - 1, "Transport and broadcast option are mutually exclusive");
-				write_cman_pipe("Transport and broadcast option are mutually exclusive");
-				return -1;
-			}
-		} else if (strcmp(str, "rdma") == 0) {
-			if (transport != TX_MECH_UDPB) {
-				transport = TX_MECH_RDMA;
-			} else {
-				snprintf(error_reason, sizeof(error_reason) - 1, "Transport and broadcast option are mutually exclusive");
-				write_cman_pipe("Transport and broadcast option are mutually exclusive");
-				return -1;
-			}
-		} else {
-			snprintf(error_reason, sizeof(error_reason) - 1, "Transport option value can be one of udp, udpb, udpu, rdma");
-			write_cman_pipe("Transport option value can be one of udp, udpb, udpu, rdma");
-			return -1;
-		}
-	}
-
 	if (add_ifaddr(objdb, mcast_name, nodename, portnum, ttl,
 		       PRIMARY_IFACE, transport)) {
 		write_cman_pipe(error_reason);
@@ -1458,6 +1478,7 @@ static int cmanpre_reloadconfig(struct objdb_iface_ver0 *objdb, int flush, const
 	hdb_handle_t cluster_parent_handle_new;
 	unsigned int config_version = 0, config_version_new = 0;
 	char *config_value = NULL;
+	char str[255];
 
 	/* don't reload if we've been told to run configless */
 	if (getenv("CMAN_NOCONFIG")) {
@@ -1469,16 +1490,16 @@ static int cmanpre_reloadconfig(struct objdb_iface_ver0 *objdb, int flush, const
 	/* find both /cluster entries */
 	objdb->object_find_create(OBJECT_PARENT_HANDLE, "cluster", strlen("cluster"), &find_handle);
 	objdb->object_find_next(find_handle, &cluster_parent_handle);
+	objdb->object_find_next(find_handle, &cluster_parent_handle_new);
+	objdb->object_find_destroy(find_handle);
 	if (!cluster_parent_handle) {
 		snprintf (error_reason, sizeof(error_reason) - 1, "Cannot find old /cluster/ key in configuration\n");
 		goto err;
 	}
-	objdb->object_find_next(find_handle, &cluster_parent_handle_new);
 	if (!cluster_parent_handle_new) {
 		snprintf (error_reason, sizeof(error_reason) - 1, "Cannot find new /cluster/ key in configuration\n");
 		goto err;
 	}
-	objdb->object_find_destroy(find_handle);
 
 	if (!objdb->object_key_get(cluster_parent_handle, "config_version", strlen("config_version"), (void *)&config_value, NULL)) {
 		if (config_value) {
@@ -1511,6 +1532,32 @@ static int cmanpre_reloadconfig(struct objdb_iface_ver0 *objdb, int flush, const
 	/* destroy the old one */
 	objdb->object_destroy(cluster_parent_handle);
 
+	/*
+	 * create cluster.cman in the new config if it doesn't exist
+	 */
+	objdb->object_find_create(cluster_parent_handle_new, "cman", strlen("cman"), &find_handle);
+	if (objdb->object_find_next(find_handle, &object_handle)) {
+		objdb->object_create(cluster_parent_handle_new, &object_handle,
+				     "cman", strlen("cman"));
+	}
+	objdb->object_find_destroy(find_handle);
+
+	/*
+	 * write cluster_id/two_node/nodename
+	 */
+	snprintf(str, sizeof(str) - 1, "%d", cluster_id);
+	objdb->object_key_create_typed(object_handle, "cluster_id",
+				       str, strlen(str) + 1, OBJDB_VALUETYPE_STRING);
+
+	if (two_node) {
+		snprintf(str, sizeof(str) - 1, "%d", 1);
+		objdb->object_key_create_typed(object_handle, "two_node",
+					       str, strlen(str) + 1, OBJDB_VALUETYPE_STRING);
+	}
+
+	objdb->object_key_create_typed(object_handle, "nodename",
+				       nodename, strlen(nodename)+1, OBJDB_VALUETYPE_STRING);
+
 	/* update the reference to the new config */
 	cluster_parent_handle = cluster_parent_handle_new;
 
