@@ -25,6 +25,7 @@
 #include <liblogthread.h>
 #include "score.h"
 #include "../daemon/cman.h"
+#include "../daemon/cnxman-socket.h"
 #include <sys/syslog.h>
 
 #define LOG_DAEMON_NAME  "qdiskd"
@@ -195,7 +196,8 @@ read_node_blocks(qd_ctx *ctx, node_info_t *ni, int max)
 			continue;
 
 		/* Unchanged timestamp: miss */
-		if (sb->ps_timestamp == ni[x].ni_last_seen) {
+		if ((sb->ps_timestamp == ni[x].ni_last_seen) &&
+		    (ni[x].ni_state != S_EXIT)) {
 			/* XXX check for average + allow grace */
 			ni[x].ni_misses++;
 			if (ni[x].ni_misses > 1) {
@@ -229,6 +231,22 @@ check_transitions(qd_ctx *ctx, node_info_t *ni, int max, memb_mask_t mask)
 
 	for (x = 0; x < max; x++) {
 
+		/*
+		   Case 0: check if master node is about to leave
+		 */
+		if (ni[x].ni_state == S_EXIT) {
+			logt_print(LOG_NOTICE, "Node %d is about to leave\n", ni[x].ni_status.ps_nodeid);
+			ni[x].ni_evil_incarnation = 0;
+			ni[x].ni_incarnation = 0;
+			ni[x].ni_seen = 0;
+			ni[x].ni_misses = 0;
+			ni[x].ni_state = S_NONE;
+			if (mask) 
+				clear_bit(mask, (ni[x].ni_status.ps_nodeid-1),
+					  sizeof(memb_mask_t));
+			continue;
+		}
+
 		/*
 		   Case 1: check to see if the node is still up
 		   according to our internal state, but has been
@@ -1267,6 +1285,50 @@ quorum_loop(qd_ctx *ctx, node_info_t *ni, int max)
 }
 
 
+/**
+  Tell the other nodes to elect a new master != me.
+ */
+static int
+quorum_reelect_master(qd_ctx *ctx, node_info_t *ni, int max)
+{
+	if (qd_write_status(ctx, ctx->qc_my_id, S_EXIT,
+			    NULL, NULL, NULL) != 0) {
+		logt_print(LOG_WARNING,
+		       "Error writing to quorum disk during reelect_master\n");
+	}
+
+	while (1) {
+		int master, x;
+		int found = 0;
+		int low_id, count;
+
+		read_node_blocks(ctx, ni, max);
+
+		for (x = 0; x < max; x++) {
+			if (ni[x].ni_state >= S_RUN) {
+				found = 1;
+			}
+		}
+
+		if (!found) {
+			logt_print(LOG_DEBUG, "No other nodes are active. Exiting\n");
+			break;
+		}
+
+		master = master_exists(ctx, ni, max, &low_id, &count);
+		if (master) {
+			logt_print(LOG_DEBUG, "New master elected: %d\n", master);
+			break;
+		}
+		/*
+		 * give time for message to be read
+		 */
+		sleep(1);
+	}
+
+	return 0;
+}
+
 /**
   Tell the other nodes we're done (safely!).
  */
@@ -1443,7 +1505,7 @@ auto_qdisk_votes(int desc)
 		logt_print(LOG_ERR, "Unable to determine qdiskd votes "
 			   "automatically\n");
 	else
-		logt_print(LOG_DEBUG, "Setting votes to %d\n", ret);
+		logt_print(LOG_DEBUG, "Setting autocalculated votes to %d\n", ret);
  
  	return (ret);
 }
@@ -1605,6 +1667,8 @@ get_dynamic_config_data(qd_ctx *ctx, int ccsfd)
 		ctx->qc_flags &= ~RF_AUTO_VOTES;
 	}
 
+	ctx->qc_auto_votes = auto_qdisk_votes(ccsfd);
+
 	snprintf(query, sizeof(query), "/cluster/quorumd/@votes");
 	if (ccs_get(ccsfd, query, &val) == 0) {
 		ctx->qc_votes = atoi(val);
@@ -1612,7 +1676,7 @@ get_dynamic_config_data(qd_ctx *ctx, int ccsfd)
 		if (ctx->qc_votes < 0)
 			ctx->qc_votes = 0;
 	} else {
-		ctx->qc_votes = auto_qdisk_votes(ccsfd);
+		ctx->qc_votes = ctx->qc_auto_votes;
 		if (ctx->qc_votes < 0) {
 			if (ctx->qc_config) {
 				logt_print(LOG_WARNING, "Unable to determine "
@@ -1878,15 +1942,21 @@ get_config_data(qd_ctx *ctx, struct h_data *h, int maxh, int *cfh)
 	*cfh = configure_heuristics(ccsfd, h, maxh,
 				    ctx->qc_interval * (ctx->qc_tko - 1));
 
-	if (*cfh) {
-		if (ctx->qc_flags & RF_MASTER_WINS) {
-			logt_print(LOG_WARNING, "Master-wins mode disabled\n");
+	if (ctx->qc_flags & RF_MASTER_WINS) {
+		if (*cfh) {
+			logt_print(LOG_WARNING, "Master-wins mode disabled "
+						"(not compatible with heuristics)\n");
+			ctx->qc_flags &= ~RF_MASTER_WINS;
+		}
+		if (ctx->qc_auto_votes != 1) {
+			logt_print(LOG_WARNING, "Master-wins mode disabled "
+						"(not compatible with more than 2 nodes)\n");
 			ctx->qc_flags &= ~RF_MASTER_WINS;
 		}
 	} else {
 		if (ctx->qc_flags & RF_AUTO_VOTES &&
-		    !(ctx->qc_flags & RF_MASTER_WINS) &&
-		    ctx->qc_votes == 1) { 
+		    !*cfh &&
+		    ctx->qc_auto_votes == 1) { 
 			/* Two node cluster, no heuristics, 1 vote for
 			 * quorum disk daemon.  Safe to enable master-wins.
 			 * In fact, qdiskd without master-wins in this config
@@ -2063,7 +2133,7 @@ main(int argc, char **argv)
 		goto out;
 	}
 
-	if (cman_start_recv_data(ch_user, qdisk_whine, 178) != 0) {
+	if (cman_start_recv_data(ch_user, qdisk_whine, CLUSTER_PORT_QDISKD) != 0) {
 		logt_print(LOG_CRIT, "Could not register with CMAN: %s\n",
 			   strerror(errno));
 		goto out;
@@ -2172,6 +2242,15 @@ main(int argc, char **argv)
 	io_nanny_start(ch_user, ctx.qc_tko * ctx.qc_interval);
 
 	if (quorum_loop(&ctx, ni, MAX_NODES_DISK) == 0) {
+		/*
+		 * if we are master and we are in master-win mode,
+		 * request other qdiskd to elect a new one
+		 */
+		if ((ctx.qc_status == S_MASTER) &&
+		    ((ctx.qc_flags & RF_MASTER_WINS) ||
+		     (ctx.qc_flags & RF_AUTO_MASTER_WINS))) {
+			quorum_reelect_master(&ctx, ni, MAX_NODES_DISK);
+		}
 		/* Only clean up if we're exiting w/o error) */
 		logt_print(LOG_NOTICE, "Unregistering quorum device.\n");
 		cman_unregister_quorum_device(ctx.qc_cman_admin);
