@@ -1013,18 +1013,6 @@ static void read_arguments(int argc, char **argv)
 	}
 }
 
-static void set_oom_adj(int val)
-{
-	FILE *fp;
-
-	fp = fopen("/proc/self/oom_adj", "w");
-	if (!fp)
-		return;
-
-	fprintf(fp, "%i", val);
-	fclose(fp);
-}
-
 int main(int argc, char **argv)
 {
 	INIT_LIST_HEAD(&domains);
@@ -1039,13 +1027,11 @@ int main(int argc, char **argv)
 			perror("main: cannot fork");
 			exit(EXIT_FAILURE);
 		}
-		umask(0);
 	}
 	lockfile();
 	init_logging();
 	log_level(LOG_INFO, "fenced %s started", RELEASE_VERSION);
 	signal(SIGTERM, sigterm_handler);
-	set_oom_adj(-16);
 
 	if (!cfgd_disable_dbus) {
 		fd_dbus_init();
