@@ -25,6 +25,8 @@
 #include <groups.h>
 #include <rg_dbus.h>
 
+#include <cpglock-internal.h>
+
 #ifdef WRAP_THREADS
 void dump_thread_states(FILE *);
 #endif
@@ -50,7 +52,7 @@ segfault(int __attribute__ ((unused)) sig)
 	char ow[64];
 	int err; // dumb error checking... will be replaced by logsys
 
-	snprintf(ow, sizeof(ow)-1, "PID %d Thread %d: SIGSEGV\n", getpid(),
+	snprintf(ow, sizeof(ow), "PID %d Thread %d: SIGSEGV\n", getpid(),
 		 gettid());
 	err = write(2, ow, strlen(ow));
 	while(1)
@@ -456,7 +458,13 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 			/* Centralized processing or request is from
 			   clusvcadm */
 			nid = event_master();
-			if (nid != my_id()) {
+			if (nid < 0) {
+				logt_print(LOG_ERR, "#40b: Unable to determine "
+					"event master\n");
+				ret = -1;
+				goto out;
+			}
+			else if (nid != my_id()) {
 				/* Forward the message to the event master */
 				forward_message(ctx, msg_sm, nid);
 			} else {
@@ -654,7 +662,11 @@ static void
 dump_internal_state(const char *loc)
 {
 	FILE *fp;
+	if (!loc)
+		return;
 	fp=fopen(loc, "w+");
+	if (!fp)
+		return;
  	dump_config_version(fp);
  	dump_threads(fp);
  	dump_vf_states(fp);
@@ -745,6 +757,14 @@ event_loop(msgctx_t *localctx, msgctx_t *clusterctx)
 	if (need_reconfigure) {
 		need_reconfigure = 0;
 		configure_rgmanager(-1, 0, NULL);
+
+		/*
+		 * A shutdown during reconfiguration would slow down
+		 * the exit request, so it's pointless to run the
+		 * deltas at this point
+		 */
+		if (shutdown_pending)
+			return 0;
 		config_event_q();
 		return 0;
 	}
@@ -778,13 +798,80 @@ cleanup(msgctx_t *clusterctx)
 }
 
 
-
 static void
 statedump(int __attribute__ ((unused)) sig)
 {
 	signalled++;
 }
 
+/*
+ * return -1 on error
+ *         0 cpglock is not needed
+ *         1 cpglock is needed
+ */
+static int
+cpglockd_needed(void)
+{
+	char *v;
+	int ccsfd;
+	int need = 0;
+	ccsfd = ccs_force_connect(NULL, 0);
+	if (ccsfd < 0)
+		return -1;
+
+
+	if (ccs_get(ccsfd,
+		    "/cluster/clusternodes/clusternode/altname/@name", &v) == 0) {
+		if (v) {
+			need = 1;
+			free(v);
+		}
+	}
+
+	if (need) {
+		if (ccs_get(ccsfd, "/cluster/totem/@rrp_mode", &v) == 0) {
+			if (v != NULL) {
+				if (!strcasecmp(v, "none")) {
+					need = 0;
+				}
+				free(v);
+			}
+		}
+	}
+
+	ccs_disconnect(ccsfd);
+	return need;
+}
+
+static int
+rgmanager_disabled(int ccsfd)
+{
+	char *v;
+	int disabled = 0;
+	int internal = 0;
+
+	if (ccsfd < 0) {
+		internal = 1;
+		ccsfd = ccs_force_connect(NULL, 0);
+		if (ccsfd < 0)
+			return -1;
+	}
+
+	if (ccs_get(ccsfd, "/cluster/rm/@disabled", &v) == 0) {
+		if (atoi(v) == 1) {
+			disabled = 1;
+			shutdown_pending = 1;
+			logt_print(LOG_NOTICE, "Resource Group Manager Disabled\n");
+		}
+		free(v);
+	}
+
+	if (internal)
+		ccs_disconnect(ccsfd);
+
+	return disabled;
+}
+
 
 /*
  * Configure logging based on data in cluster.conf
@@ -806,6 +893,8 @@ configure_rgmanager(int ccsfd, int dbg, int *token_secs)
 
 	setup_logging(ccsfd);
 
+	rgmanager_disabled(ccsfd);
+
 	if (token_secs && ccs_get(ccsfd, "/cluster/totem/@token", &v) == 0) {
 		tmp = atoi(v);
 		if (tmp >= 1000) {
@@ -944,14 +1033,15 @@ int
 main(int argc, char **argv)
 {
 	int rv, do_init = 1;
-	char foreground = 0, wd = 1;
+	int cpg_locks = 0;
+	char foreground = 0, wd = 1, cpg_lock_opt = 0;
 	cman_node_t me;
 	msgctx_t *cluster_ctx;
 	msgctx_t *local_ctx;
 	pthread_t th;
 	cman_handle_t clu = NULL;
 
-	while ((rv = getopt(argc, argv, "wfdNq")) != EOF) {
+	while ((rv = getopt(argc, argv, "wfdNqC::")) != EOF) {
 		switch (rv) {
 		case 'w':
 			wd = 0;
@@ -968,6 +1058,24 @@ main(int argc, char **argv)
 		case 'q':
 			rgm_dbus_notify = 0;
 			break;
+		case 'C':
+			if (optarg) {
+				if (!strcmp(optarg, "0")) {
+					/* force disable */
+					cpg_lock_opt = -1;
+				} else if (!strcmp(optarg, "1")) {
+					/* force enable */
+					cpg_lock_opt = 1;
+				} else {
+					fprintf(stderr,
+						"Unknown argument for -C: argument must be 1 or 0\n");
+					return -1;
+				}
+			} else {
+				/* auto handling: enable when RRP mode is enabled */
+				cpg_lock_opt = 0;
+			}
+			break;
 		default:
 			return 1;
 			break;
@@ -978,6 +1086,13 @@ main(int argc, char **argv)
 		debug = 1;
 	}
 
+	/* If we're disabled in the configuration, don't fork */
+	if (rgmanager_disabled(-1) > 0) {
+		fprintf(stderr,
+			"rgmanager disabled in configuration; not starting\n");
+		return 2;
+	}
+
 	if (!foreground && (geteuid() == 0)) {
 		daemon_init(argv[0]);
 		if (wd && !debug && !watchdog_init())
@@ -1007,10 +1122,33 @@ main(int argc, char **argv)
 		return -1;
 	}
 
-	if (clu_lock_init(rgmanager_lsname) != 0) {
-		printf("Locks not working!\n");
-		cman_finish(clu);
-		return -1;
+	xmlInitParser();
+
+	cpg_locks = 0;
+	if (cpg_lock_opt == 1) { /* force enable */
+		cpg_locks = 1;
+	} else if (cpg_lock_opt == 0) { /* autodetect */
+		cpg_locks = cpglockd_needed();
+		if (cpg_locks < 0) {
+			printf("Unable to determine if cpglock is required!\n");
+			cman_finish(clu);
+			return -1;
+		}
+	}
+
+	if (!cpg_locks) {
+		if (clu_lock_init(rgmanager_lsname) != 0) {
+			printf("Locks not working!\n");
+			cman_finish(clu);
+			return -1;
+		}
+	} else {
+		logt_print(LOG_INFO, "Using CPG for locking (EXPERIMENTAL)\n");
+		if (cpg_lock_initialize() != 0) {
+			printf("Locks not working!\n");
+			cman_finish(clu);
+			return -1;
+		}
 	}
 
 	memset(&me, 0, sizeof(me));
@@ -1035,6 +1173,9 @@ main(int argc, char **argv)
 	   read the resource group trees from ccsd.
 	 */
 	configure_rgmanager(-1, debug, &cluster_timeout);
+	if (shutdown_pending == 1)
+		goto out_ls;
+
 	logt_print(LOG_NOTICE, "Resource Group Manager Starting\n");
 
 	if (rgm_dbus_notify && rgm_dbus_init() != 0) {
@@ -1114,6 +1255,7 @@ main(int argc, char **argv)
 	if (rg_initialized())
 		cleanup(cluster_ctx);
 	rv = 0;
+	xmlCleanupParser();
 out_ls:
 	clu_lock_finished(rgmanager_lsname);
 
